/**
 * This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * Created on Jan 24, 2003
 * Copyright (C) 2002  Aditisoft Inc
 */
package com.aditisoft.editor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author Anil K Patel
 * 
 * This class is used to trnsfer informations of page from CMS to Applet.
 */
public class CmsDocument {

	/** Name of the master template */
	String mMasterTemplateName = "";
	/** Title of the master template */
	String mMasterTemplateTitle = "";
	/** Title of the page */
	String mDocumentTitle = "";

	/** List of all available master templates. Stored as title,name pairs */
	Hashtable mMasterTemplateList = new Hashtable();
	/** Body contents of page */
	String mDocumentBody = "";

	/**
	 * Empty Constructor CmsDocument.
	 * @param in InputStream
	 */
	private CmsDocument(InputStream in) {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		DocumentBuilder db = null;
		try {
			db = dbf.newDocumentBuilder();
			Document doc = db.parse(in);
			Element root = (Element) doc.getElementsByTagName("CMSDocument").item(0);
			setMasterTemplate(root);
			setDocumentBody(root);
			setMasterTemplateList(root);
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		catch (SAXException e) {
			e.printStackTrace();
		}
		catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Method getDocument.
	 * @param doc Completer string representation of this document.
	 * @return CmsDocument
	 */
	public static CmsDocument getDocument(String doc) {
		CmsDocument cmsDoc = new CmsDocument(new ByteArrayInputStream(doc.getBytes()));
		return cmsDoc;
	}

	/**
	 * Method setMasterTemplate sets the used master template of document.
	 * @param title title of master template.
	 */
	public void setMasterTemplate(String title) {
		String template = (String) mMasterTemplateList.get(title);
		if (template != null) {
			mMasterTemplateName = template;
			mMasterTemplateTitle = title;
		}
	}

	/**
	 * Returns the mDocumentBody body content of page.
	 * @return String
	 */
	public String getDocumentBody() {
		return mDocumentBody;
	}

	/**
	 * Returns the mDocumentTitle title of page.
	 * @return String
	 */
	public String getDocumentTitle() {
		return mDocumentTitle;
	}

	/**
	 * Returns the mMasterTemplateName name of used master template.
	 * @return String
	 */
	public String getMasterTemplateName() {
		return mMasterTemplateName;
	}

	/**
	 * Returns the mMasterTemplateTitle title of used master template.
	 * @return String
	 */
	public String getMasterTemplateTitle() {
		return mMasterTemplateTitle;
	}

	/**
	 * Sets the mDocumentBody.
	 * @param mDocumentBody The mDocumentBody to set
	 */
	public void setDocumentBody(String documentBody) {
		mDocumentBody = documentBody;
	}

	/**
	 * Method setDocumentBody sets body contains of page.
	 * @param root Element containing the body contains. 
	 */
	public void setDocumentBody(Element root) {
		Element node = (Element) root.getElementsByTagName("DocumentBody").item(0);
		if (node != null) {
			mDocumentBody = node.getChildNodes().item(0).getNodeValue();
			mDocumentTitle = node.getAttribute("Title");
		}
	}

	/**
	 * Sets the mDocumentTitle.
	 * @param mDocumentTitle The mDocumentTitle to set
	 */
	public void setDocumentTitle(String documentTitle) {
		mDocumentTitle = documentTitle;
	}

	/**
	* Sets the mMasterTemplateName.
	* @param mMasterTemplateName The mMasterTemplateName to set
	*/
	public void setMasterTemplateName(String masterTemplateName) {
		mMasterTemplateName = masterTemplateName;
	}

	/**
	 * Sets the mMasterTemplateTitle.
	 * @param mMasterTemplateTitle The mMasterTemplateTitle to set
	 */
	public void setMasterTemplateTitle(String masterTemplateTitle) {
		mMasterTemplateTitle = masterTemplateTitle;
	}

	/**
	 * Method setBodyTitle.
	 * @param root Element containing the name and title of used master template. 
	 */
	public void setMasterTemplate(Element root) {
		Element node = (Element) root.getElementsByTagName("MasterTemplate").item(0);
		if (node != null) {
			mMasterTemplateName = node.getAttribute("Name");
			mMasterTemplateTitle = node.getAttribute("Title");
		}
	}

	private String getNodeValue(Element root, String nodeName) {
		String returnVal = "";
		NodeList list = root.getElementsByTagName(nodeName);
		if (list != null && list.getLength() > 0) {
			Element node = (Element) root.getElementsByTagName(nodeName).item(0);
			returnVal = node.getChildNodes().item(0).getNodeValue();
		}
		return returnVal;
	}

	/**
	 * Method setMasterTemplateList sets the list of all available master templates
	 * as pair of title and name.
	 * @param root
	 */
	public void setMasterTemplateList(Element root) {
		Element node = (Element) root.getElementsByTagName("MasterTemplateOptionList").item(0);
		NodeList list = node.getChildNodes();
		if (list != null && list.getLength() > 0) {
			for (int i = 0; i < list.getLength(); i++) {
				Element item = (Element) list.item(i);
				String name = item.getAttribute("Name");
				String title = item.getAttribute("Title");
				mMasterTemplateList.put(title, name);
			}
		}
	}

	/**
	 * Method getAllTemplates returns the titles of all available master templates.
	 * @return Vector
	 */
	public Vector getAllTemplates() {
		Vector allTemplates = new Vector();
		Enumeration titles = mMasterTemplateList.keys();
		while (titles.hasMoreElements()) {
			allTemplates.add(titles.nextElement());
		}
		return allTemplates;
	}

}
