// This code has been adoped from Curriculum Development Tool
//Copyright:    Copyright (c) 2001
//Author:       David Bradford - dbrad@medstat.med.utah.edu
//Company:      Knowledge Weavers - http://medstat.med.utah.edu/kw/

package com.aditisoft.editor;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.util.Arrays;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Iterator;

import javax.swing.JOptionPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import com.hexidec.ekit.action.ListAutomationAction;

/**
 * This class performs the actions that take place in the WYSIWYG editor.
 * These functions were very hard to get written and some of them do not work
 * in the best possible way.  If you can find a way to improve any of them go ahead.
 * At the beginning of most the the functions is the code:<br><pre>
 * if(parent.isClosed()) {
 *     return;
 * }
 * </pre><br>This code is to make sure that there is a file open to edit before the
 * command is executed.
 *
 */

/**
 * WYSIWYG editor for HTML content. 
 * @version 1.0
 * @author Anil K Patel
 */

public class EditorActions {

	/** The JTextPane that is being edited. */
	private HTMLEditor parent;
	/** The EditorKit used by the JTextPane. */
	private HTMLEditorKit htmlKit;

	/**
	 * Creates an instance of EditorActions.
	 *
	 * @param parent JTextPane that is to be edited.
	 */
	public EditorActions(HTMLEditor parent) {
		this.parent = parent;
		htmlKit = (HTMLEditorKit) parent.getEditorKit();
	}

	/**
	 * Performs the undo command.
	 */
	public void undo() {
		parent.undo();
	}

	/**
	 * Preforms the redo command.
	 */
	public void redo() {
		parent.redo();
	}

	/**
	 * Changes the size of the selected text.
	 *
	 * @param size New size of text.  This size is specified like the font tag. ie, +1, +2, ...
	 */
	public void setSize(String size) {
		//check if there is any selected text
		String s = parent.getSelectedText();
		if (s == null) {
			return;
		}
		new StyledEditorKit.FontSizeAction("Fontsize", Integer.parseInt(size)).actionPerformed(
			new ActionEvent(parent, 0, "Fontsize"));
	}

	/**
	 * Changes the color of the selected text.
	 *
	 * @param col New color of text.
	 */
	public void setColor(Color col) {
		new StyledEditorKit.ForegroundAction("Color", col).actionPerformed(new ActionEvent(parent, 0, "Color"));
		parent.requestFocus();
	}

	public void setFont(String family) {
		String s = parent.getSelectedText();
		if (s == null) {
			return;
		}
		new StyledEditorKit.FontFamilyAction("Fontfamily", family).actionPerformed(new ActionEvent(parent, 0, family));
	}

	/**
	 * Makes the selected text bold.
	 */
	public void setBold() {
		new StyledEditorKit.BoldAction().actionPerformed(new ActionEvent(parent, 0, "Bold"));
		parent.requestFocus();
	}

	/**
	 * Makes the selected text italic.
	 */
	public void setItalic() {
		new StyledEditorKit.ItalicAction().actionPerformed(new ActionEvent(parent, 0, "Italic"));
		parent.requestFocus();
	}

	/**
	 * Makes the selected text underlined.
	 */
	public void setUnderline() {
		new StyledEditorKit.UnderlineAction().actionPerformed(new ActionEvent(parent, 0, "Underline"));
		parent.requestFocus();
	}

	/**
	 * Makes the selected text a subscript.
	 */
	public void setSubscript() {
		new SubscriptAction().actionPerformed(new ActionEvent(parent, 0, SubscriptAction.SUBSCRIPT_ACTION));
		parent.requestFocus();
	}

	/**
	 * Makes the selected text a superscript.
	 */
	public void setSuperscript() {
		new SuperscriptAction().actionPerformed(new ActionEvent(parent, 0, SuperscriptAction.SUPERSCRIPT_ACTION));
		parent.requestFocus();
	}

	/**
	 * Adds a horizontal rule at the cursor position or at the end of a selection.
	 */
	public void setHorizontal() {
		int caretPos = parent.getSelectionEnd();
		try {
			htmlKit.insertHTML((HTMLDocument) parent.getDocument(), caretPos, "<hr>", 0, 0, HTML.Tag.HR);
		}
		catch (Exception e) {
		}
		parent.requestFocus();
	}

	/**
	 * Adds a paragraph to at the cursor.  This function uses the default action for the
	 * enter key to add the paragraph.  So the paragraph action must be set using the
	 * keymap for the JTextPane.
	 *
	 * @see cdt.editor.EditorToolbar EditorToolbar
	 */
	public void setParagraph() {
		new HTMLEditorKit.InsertHTMLTextAction("Paragraph", "<P>", HTML.Tag.BODY, HTML.Tag.P).actionPerformed(
			new ActionEvent(parent, 0, "Paragraph"));
		parent.requestFocus();
	}

	/**
	 * Adds a &lt;br&gt; tag at the cursor.
	 */
	public void setBreak() {
		int caretPos = parent.getSelectionEnd();
		//        String htmlText = parent.getText();

		try {
			htmlKit.insertHTML((HTMLDocument) parent.getDocument(), caretPos, "<br></ol></ul>", 0, 0, HTML.Tag.BR);
		}
		catch (Exception e) {
		}

		//        htmlText = htmlText.substring(0, caretPos+1) + "</ol></ul><br>" + htmlText.substring(caretPos+1) ;
		//        parent.setText(htmlText);

		if (caretPos == 1) {
			// I'm not sure why, but if the caretPos == 1 and you
			// setBreak();, then no changes will show up until
			// parent.reset(); is called.  No need to have it reset()
			// every time, though. -Brad
			parent.reset();
		}
		parent.setCaretPosition(caretPos + 1);
		parent.requestFocus();
	}

	/**
	 * Adds a non-breaking space at the cursor.
	 */
	public void setNonBreak() {
		parent.replaceSelection("\u00A0");
		parent.requestFocus();
	}

	/**
	 * Left aligns the paragraph in which the cursor is located.
	 */
	public void setLeft() {
		new StyledEditorKit.AlignmentAction("Left", 0).actionPerformed(new ActionEvent(parent, 0, "Left"));
		parent.requestFocus();
	}

	/**
	 * Centers the paragraph in which the cursor is located.
	 */
	public void setCenter() {
		new StyledEditorKit.AlignmentAction("Center", 1).actionPerformed(new ActionEvent(parent, 0, "Center"));
		parent.requestFocus();
	}

	/**
	 * Right aligns the paragraph in which the cursor is located.
	 */
	public void setRight() {
		new StyledEditorKit.AlignmentAction("Right", 2).actionPerformed(new ActionEvent(parent, 0, "Right"));
		parent.requestFocus();
	}

	/**
	 * Makes the text that is selected a link.  This function uses the LinkDialog class to
	 * tell where to set the link to.
	 *
	 * @see cdt.editor.LinkDialog LinkDialog
	 */
	public void setLink() {
		if (null == parent.getSelectedText()) {
			return;
		}
		String name = "";
		// Tries to find out if the selected text already has a link
		// and if so, feed it to the LinkDialog.
		AttributeSet mas = htmlKit.getInputAttributes();
		mas = (AttributeSet) mas.getAttribute(HTML.Tag.A);
		if (null != mas) {
			String temp = (String) mas.getAttribute(HTML.Attribute.HREF);
			if (null != temp) {
				name = temp;
			}
		}
		//        int deep = cdt.projects.Project.currentProject.getDepth();
		//		LinkDialog lDlg = new LinkDialog(cdt.Frame1.frame, deep, name);
		//		name = lDlg.getLink();

		// Anil: Use a simple dialog and get User to key in link value
		name = "http://www.yahoo.com";

		new LinkAction(name).actionPerformed(new ActionEvent(parent, 0, LinkAction.LINK_ACTION));
		parent.requestFocus();
	}

	/**
	 * This function increase the indent of the paragraph the caret is in.
	 */
	public void addIndent() {
		if (parent.isClosed()) {
			return;
		}
		new IndentAction(IndentAction.RIGHT).actionPerformed(new ActionEvent(parent, 0, IndentAction.RIGHT));
		parent.requestFocus();
	}

	/**
	 * This function decreases the indent or the paragraph the caret is in.
	 */
	public void removeIndent() {
		if (parent.isClosed()) {
			return;
		}
		new IndentAction(IndentAction.LEFT).actionPerformed(new ActionEvent(parent, 0, IndentAction.LEFT));
		parent.requestFocus();
	}

	/**
	 * Creates an Ordered List.  */

	public void setOL() {
		if (null == parent.getSelectedText()) {
			return;
		}
		new ListAutomationAction(parent, "Numbered List", HTML.Tag.OL).actionPerformed(
			new ActionEvent(parent, 0, "NumberedList"));
	}

	/**
	 * Creates an unordered List.  */

	public void setUL() {
		if (null == parent.getSelectedText()) {
			return;
		}
		new ListAutomationAction(parent, "Bulleted List", HTML.Tag.UL).actionPerformed(
			new ActionEvent(parent, 0, "BulletedList"));
	}

	/**
	 * Creates a table and adds it where the caret is located.  This function uses the
	 * TableWizard to determine what the table should look like.
	 */
	public void setTable() {
		if (parent.isClosed()) {
			return;
		}
		TableWizard tableWiz = new TableWizard(JOptionPane.getFrameForComponent(parent));
		WizardData d = tableWiz.getData();
		if (d != null) {
			String s = (String) d.remove("tag");

			new HTMLEditorKit.InsertHTMLTextAction("Table", s, HTML.Tag.BODY, HTML.Tag.TABLE).actionPerformed(
				new ActionEvent(parent, 0, "TABLE"));
		}
		parent.requestFocus();
	}

	/**
	 * This function is here to remove tags from html.  This is needed because for
	 * some reason when pasting for Microsoft Word the text that is in the clipboard
	 * has some weird, unnecessary tags.
	 *
	 * @param html Html to strip tags from.
	 * @param tagList Tags to remove from the html.
	 * @return Html with all the tags specified removed.
	 */
	private StringBuffer removeTags(StringBuffer html, java.util.List tagList) {
		/* Trying to speed things up.. hoping this will help */
		StringBuffer search = new StringBuffer(html.toString().toLowerCase());
		Iterator i = tagList.iterator();
		while (i.hasNext()) {
			String tag = (String) i.next();
			int start = search.lastIndexOf(tag);
			while (start != -1) {
				int end = search.indexOf(">", start);
				html.delete(start, end + 1);
				search.delete(start, end + 1);
				start = search.lastIndexOf(tag, start - 1);
			}
		}
		return html;
	}

	/**
	 * These are used to keep formatting when copying and pasting within the application.
	 * For some reason the parent.copy function only copies the text and not the format.
	 * How I'm doing it is whenever something is copied, the text and the format
	 * are saved and then when something is pasted, if it is the same as the last text
	 * copied from the program it will apply the style to it.
	 *
	 * miniClipboard contains the text of the stuff in the clipboard.
	 */
	private static String miniClipboard = "";
	/** miniClipStyle contains the attributes of the stuff in the clipbard. */
	private static SimpleAttributeSet[] miniClipStyle = null;

	/**
	 * This copies the selected text.  Since the formatting is not copied, we have to
	 * save the formating and the text in static variables and then when something is
	 * pasted we check whether the text is the same.  If it is we apply the saved style
	 * to it.
	 */
	//	public void copy() {
	//		if (parent.isClosed()) {
	//			return;
	//		}
	//		StringBuffer sb = new StringBuffer();
	//		int start = parent.getSelectionStart();
	//		int end = parent.getSelectionEnd();
	//		miniClipStyle = new SimpleAttributeSet[end - start + 1];
	//		for (int i = start; i < end; i++) {
	//			try {
	//				parent.moveCaretPosition(i);
	//				sb.append(parent.getStyledDocument().getText(i, 1));
	//				miniClipStyle[i - start] = new SimpleAttributeSet(parent.getCharacterAttributes());
	//
	//			}
	//			catch (Exception e) {
	//				e.printStackTrace();
	//			}
	//		}
	//		miniClipboard = sb.toString();
	//		parent.select(start, end);
	//
	//		parent.copy();
	//		parent.requestFocus();
	//	}

	/**
	 * Cuts the selected text.  This function is really simple compared to the other
	 * clipboard functions.  It calls copy(), and then replaceSelection(""). Amazing Right?
	 */
	//	public void cut() {
	//		if (parent.isClosed()) {
	//			return;
	//		}
	//		copy();
	//		parent.replaceSelection("");
	//		parent.requestFocus();
	//	}

}