//Title:        Curriculum Development Tool
//Copyright:    Copyright (c) 2001
//Author:       David Bradford - dbrad@medstat.med.utah.edu
//Company:      Knowledge Weavers - http://medstat.med.utah.edu/kw/
//File:         editor/TableViewer.java
//Description:  Component to view a table

package com.aditisoft.editor;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.util.Vector;

/**
 * This is a component used to view a table.  It allows you to add/subtract rows and columns,
 * select boxes in the table, and span rows and columns.  After several tries to get this
 * working correctly  this is what I came up with.  How I keep the information of the table is
 * by using a 2d vector that stores how many columns and rows the that cell at each position in
 * the vector spans.  So for a normal 2x2 table the vector looks like this:<br><br><pre>
 * -------------<br>
 * |  1  |  1  |<br>
 * -------------<br>
 * |  1  |  1  |<br>
 * -------------<br><br></pre>
 * And this seems to work better than any thing else I have been able to think of.
 *
 * @version 1.0
 * @author David Bradford
 */
public class TableViewer extends Canvas {
	/** 2-d vector representation of the table. */
	private Vector2d table;
	/** Number of columns in the table. */
	private int columns;
	/** Number of rows in the table. */
	private int rows;
	/** Which row is currently selected. */
	private int selectedRow = -1;
	/** Which column is currently selected. */
	private int selectedCol = -1;
	/** Height of table viewer. */
	final private int TABLE_HEIGHT = 145;
	/** Width of table viewer. */
	final private int TABLE_WIDTH = 225;

	/**
	 * Constructs a TableViewer with 2 rows and 2 columns.
	 */
	public TableViewer() {
		this(2, 2);
	}

	/**
	 * Constructs a TableViewer with rows and columns specified by the user.
	 *
	 * @param columns Number of columns in viewer.
	 * @param rows Number of rows in viewer.
	 */
	public TableViewer(int columns, int rows) {
		table = new Vector2d(columns, rows);
		for (int i = 0; i < columns; i++) {
			for (int j = 0; j < rows; j++) {
				table.add(j, i, new Cell());
			}
		}
		this.columns = columns;
		this.rows = rows;

		setSize(TABLE_WIDTH, TABLE_HEIGHT);
		addMouseListener(new CellMouseListener(this));
	}

	/**
	 * Adds one column to the table.
	 */
	public void addCol() {
		columns++;
		for (int i = 0; i < rows; i++) {
			table.add(i, columns - 1, new Cell());
		}
		repaint();
	}

	/**
	 * Removes one columns from the table.  If there is only one column in the table
	 * nothing happens.
	 */
	public void removeCol() {
		if (columns > 1) {
			columns--;
			repaint();
		}
	}

	/**
	 * Adds one to the span of the column of the selected cell.
	 */
	public void spanCol() {
		if (selectedRow != -1 && selectedCol != -1) {
			Cell c = (Cell) table.elementAt(selectedRow, selectedCol);
			if (selectedCol + c.getColSpan() < columns) {
				Cell c2 =
					(Cell) table.elementAt(
						selectedRow,
						selectedCol + c.getColSpan());
				if (c.getRowSpan() != c2.getRowSpan()) {
					return;
				}
				table.add(
					selectedRow,
					selectedCol,
					new Cell(c.getColSpan() + c2.getColSpan(), c.getRowSpan()));
				table.add(
					selectedRow,
					selectedCol + c.getColSpan(),
					new Cell(0, 0));
				repaint();
			}
		}
	}

	/**
	 * Adds one row to the table.
	 */
	public void addRow() {
		rows++;
		for (int i = 0; i < columns; i++) {
			table.add(rows - 1, i, new Cell());
		}
		repaint();
	}

	/**
	 * Removes one row from the table.  If there is only one row does nothing.
	 */
	public void removeRow() {
		if (rows > 1) {
			rows--;
			repaint();
		}
	}

	/**
	 * Adds one to the span of the row of the selected cell.
	 */
	public void spanRow() {
		if (selectedRow != -1 && selectedCol != -1) {
			Cell c = (Cell) table.elementAt(selectedRow, selectedCol);
			if (selectedRow + c.getRowSpan() < rows) {
				Cell c2 =
					(Cell) table.elementAt(
						selectedRow + c.getRowSpan(),
						selectedCol);
				if (c.getColSpan() != c2.getColSpan()) {
					return;
				}
				table.add(
					selectedRow,
					selectedCol,
					new Cell(c.getColSpan(), c.getRowSpan() + c2.getRowSpan()));
				table.add(
					selectedRow + c.getRowSpan(),
					selectedCol,
					new Cell(0, 0));
				repaint();
			}
		}
	}

	/**
	 * Gets which cell should be selected from the position the user clicked the mouse.
	 *
	 * @param x X-coordinate of user click.
	 * @param y Y-coordinate of user click.
	 */
	private void select(int x, int y) {
		final int lineSize = 2;
		final int cellWidth =
			(TABLE_WIDTH - lineSize * (columns + 1)) / columns;
		final int cellHeight = (TABLE_HEIGHT - lineSize * (rows + 1)) / rows;
		int lastX = 0;
		int nextX = 0;
		int lastY = 0;
		for (int i = 0; i < columns; i++) {
			for (int j = 0; j < rows; j++) {
				Cell c = (Cell) table.elementAt(j, i);
				if (x < cellWidth * (i) + cellWidth * c.getColSpan()
					&& x > lastX) {
					if (y < cellHeight * (j) + cellHeight * c.getRowSpan()
						&& y > lastY) {
						selectedRow = j;
						selectedCol = i;
						repaint();
						return;
					}
				}
				lastY = cellHeight * (j) + cellHeight * c.getRowSpan();
				nextX = cellWidth * i + cellWidth * c.getColSpan();
			}
			lastX = nextX;
			lastY = 0;
		}
		repaint();
	}

	/**
	 * Converts table to code to create an html table.  This function does not include the
	 * table tag.  It only contains the td and tr tags.
	 *
	 * @return html code for a table.
	 */
	public String toString() {
		Vector[] v = new Vector[rows];
		for (int i = 0; i < rows; i++) {
			v[i] = new Vector();
		}
		StringBuffer s = new StringBuffer();
		for (int i = 0; i < rows; i++) {
			s.append("<tr>");
			for (int j = 0; j < columns;) {
				if (v[i].contains(new String("" + j))) {
					j++;
				} else {
					Cell c = (Cell) table.elementAt(i, j);
					s.append("\n<td");
					if (c.getColSpan() > 1) {
						s.append(" colspan=\"" + c.getColSpan() + "\"");
					}
					if (c.getRowSpan() > 1) {
						s.append(" rowspan=\"" + c.getRowSpan() + "\"");
						for (int r = 1; r < c.getRowSpan(); r++) {
							for (int col = 0; col < c.getColSpan(); col++) {
								v[i + r].add(new String("" + (col + j)));
							}
						}
					}
					s.append("></td>");
					j += c.getColSpan();
				}
			}
			s.append("\n</tr>\n");
		}
		return s.toString();
	}

	/**
	 * Overloaded to draw the table viewer.
	 *
	 * @param g Graphics to draw on.
	 */
	public void paint(Graphics g) {
		BufferedImage i =
			new BufferedImage(
				TABLE_WIDTH,
				TABLE_HEIGHT,
				BufferedImage.TYPE_INT_RGB);
		draw(i.getGraphics());
		g.drawImage(i, 0, 0, null);
	}

	/**
	 * Draws the table viewer.
	 *
	 * @param g Graphics to draw on.
	 */
	private void draw(Graphics g) {
		final int lineSize = 2;
		final int cellWidth =
			(TABLE_WIDTH - lineSize * (columns + 1)) / columns;
		final int cellHeight = (TABLE_HEIGHT - lineSize * (rows + 1)) / rows;

		g.setColor(Color.white);
		g.fillRect(0, 0, TABLE_WIDTH, TABLE_HEIGHT);

		g.setColor(Color.gray);
		for (int i = 0; i < columns; i++) {
			for (int j = 0; j < rows; j++) {
				if (i == selectedCol && j == selectedRow) {
					g.setColor(Color.blue);
				}
				Cell c = (Cell) table.elementAt(j, i);
				int rowS = c.getRowSpan();
				int colS = c.getColSpan();
				int startX = i * cellWidth + i * lineSize;
				int startY = j * cellHeight + j * lineSize;
				int endX = cellWidth * colS + (colS) * lineSize;
				int endY = cellHeight * rowS + (rowS) * lineSize;
				g.fillRect(
					startX + lineSize,
					startY + lineSize,
					endX - lineSize,
					endY - lineSize);
				g.setColor(Color.gray);
			}
		}
		if (selectedCol != -1 && selectedRow != -1) {
			g.setColor(Color.blue);
			Cell c = (Cell) table.elementAt(selectedRow, selectedCol);
			int rowS = c.getRowSpan();
			int colS = c.getColSpan();
			int startX = selectedCol * cellWidth + selectedCol * lineSize;
			int startY = selectedRow * cellHeight + selectedRow * lineSize;
			int endX = cellWidth * colS + (colS) * lineSize;
			int endY = cellHeight * rowS + (rowS) * lineSize;
			g.fillRect(
				startX + lineSize,
				startY + lineSize,
				endX - lineSize,
				endY - lineSize);
		}
	}

	/**
	 * Class that represents a cell in the table.
	 */
	private class Cell {
		/** Number of columns this cell spans. */
		private int colSpan = 1;
		/** Number of rows this cell spans. */
		private int rowSpan = 1;

		/**
		 * Creates a cell that spans 1 column and 1 row.
		 */
		public Cell() {
		}

		/**
		 * Creates a cell that lets the user specify the rows and columns spanned.
		 *
		 * @param colSpan Number of columns spanned.
		 * @param rowSpan Number of rows spanned.
		 */
		public Cell(int colSpan, int rowSpan) {
			this.colSpan = colSpan;
			this.rowSpan = rowSpan;
		}

		/**
		 * Gets the number of columns spanned.
		 *
		 * @return Number of columns spanned.
		 */
		public int getColSpan() {
			return colSpan;
		}

		/**
		 * Gets the number of rows spanned.
		 *
		 * @return Number of rows spanned.
		 */
		public int getRowSpan() {
			return rowSpan;
		}
	}

	/**
	 * Listens for user to select a cell.
	 */
	private class CellMouseListener implements MouseListener {
		/** TableViewer that is being clicked on.*/
		private final TableViewer tv;

		/**
		 * Creates a CellMouseListener for the given TableViewer.
		 *
		 * @param tv TableViewer that is to be watched for click.
		 */
		public CellMouseListener(TableViewer tv) {
			this.tv = tv;
		}

		/**
		 * Selects the cell the user click on.
		 *
		 * @param event Infomation about the click.
		 */
		public void mouseClicked(MouseEvent event) {
			tv.select(event.getX(), event.getY());
		}

		/**
		 *  Defined to implement MouseListener
		 *
		 * @param event Not used.
		 * */
		public void mouseEntered(MouseEvent event) {
		}
		/**
		 * Defined to implement MouseListener
		 *
		 * @param event Not used.
		 */
		public void mouseExited(MouseEvent event) {
		}
		/**
		 * Defined to implement MouseListener
		 *
		 * @param event Not used.
		 */
		public void mousePressed(MouseEvent event) {
		}
		/**
		 * Defined to implement MouseListener
		 *
		 * @param event Not used.
		 */
		public void mouseReleased(MouseEvent event) {
		}
	}

}
