//Title:        Wizard Frame
//Copyright:    Copyright (c) 2001
//Author:       David Bradford - bradford@cs.utah.edu
//File:         WizardFrame.java
//Description:  Frame for Wizard

package com.aditisoft.editor;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.Stack;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A particial implementation of the Wizard interface.  The <code>WizardFrame
 * </code> class should be inherited from in order to create a wizard.
 *
 * @version 1.0
 * @author David Bradford<BR>
 *		   Brad Schaefer (<A HREF="mailto:schaefer@medstat.med.utah.edu">schaefer@medstat.med.utah.edu</A>)
 */
public abstract class WizardFrame extends JDialog {
	/**
	 * Keeps all of the data for wizards in a {@link cdt.wizard.WizardData WizardData} object
	 * which is just a wrapper for a HashTable.
	 */
	protected static WizardData data = new WizardData();

	/**
	 * Contains the frames that this wizard has progressed through.  Gets reset when the
	 * finish button is pressed using {@link #purgeFrames() purgeFrames()}.
	 */
	protected static Stack frames = new Stack();

	/** This is a LinkedList of the keys that are not needed after the wizard is finished */
	private static LinkedList purgeableKeys = new LinkedList();

	/** JPanel that contains the 'meat' of the wizard. */
	protected JPanel centerPanel = new JPanel();
	private JButton previousButton = new JButton("< Previous");
	private JButton nextButton = new JButton("Next >");
	private JButton cancelButton = new JButton("Cancel");
	private JButton finishButton = new JButton("Finish");
	private JLabel label1 = new JLabel();
	private JLabel label2 = new JLabel();

	/**
	 * Creates a new WizardFrame.
	 *
	 * @param parent The JFrame that this wizard appears on top of.
	 */
	public WizardFrame(Frame parent) {
		//		super(parent);
		super();
//		data.put("parentFrame", parent);
		data.put("cancelled", "false");
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	}

	/**
	 * Draws the basic elements of the wizard frame that all the frames will
	 * have in common.  This function needs to be called in order for the
	 * wizard to display correctly.
	 */
	protected void init() {
		customize();
		getContentPane().setLayout(new BorderLayout());
		JPanel topPanel = new JPanel(new GridLayout(2, 1));
		JPanel bottomPanel = new JPanel();

		label1.setHorizontalAlignment(JLabel.CENTER);
		label2.setHorizontalAlignment(JLabel.CENTER);

		topPanel.add(label1);
		topPanel.add(label2);

		ButtonHandler bh = new ButtonHandler();
		previousButton.addActionListener(bh);
		bottomPanel.add(previousButton);
		nextButton.addActionListener(bh);
		bottomPanel.add(nextButton);
		cancelButton.addActionListener(bh);
		bottomPanel.add(cancelButton);
		finishButton.addActionListener(bh);
		bottomPanel.add(finishButton);

		getContentPane().add(topPanel, BorderLayout.NORTH);
		getContentPane().add(centerPanel, BorderLayout.CENTER);
		getContentPane().add(bottomPanel, BorderLayout.SOUTH);

		setResizable(false);

		pack();

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();
		if (frameSize.height > screenSize.height)
			frameSize.height = screenSize.height;
		if (frameSize.width > screenSize.width)
			frameSize.width = screenSize.width;
		setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

		setModal(true);
		setVisible(true);
	}

	/**
	 * Gets the information retrieved by this wizard.
	 * @return Collection of Data wizard has retrieved or <code>null</code>
	 *         if wizard was cancelled.
	 */
	public WizardData getData() {
		String canceled = (String) data.get("cancelled");
		if (canceled.equals("true")) {
			return null;
		}
		return data;
	}

	/**
	 * Adds a piece of information to the data stored by wizard.
	 * @param key Name to store information under.
	 * @param value Information to store.
	 */
	public void putData(String key, Object value) {
		data.put(key, value);
	}

	/**
	 * Removes a piece of information stored by the wizard.
	 *
	 * @param key Key to be deleted.
	 * @return The object stored in this WizardData corresponding to
	 *      the key.
	 */
	public Object removeData(String key) {
		return data.remove(key);
	}

	/**
	 * Adds a key that will be purged when {@link #resetData() resetData()} is called.
	 *
	 * @param key Key to the {@link cdt.wizard.WizardData WizardData} HashMap that will
	 *		be marked as "purgeable".  This function is used to enforce the fact that
	 *		our LinkedList only contains Strings.  No other function should attempt to
	 *		directly add to the {@link #purgeableKeys purgeableKeys} variable.
	 */
	protected void addPurgeableKey(String key) {
		if (!purgeableKeys.contains(key)) {
			purgeableKeys.add(key);
		}
	}

	/**
	 * Empties the {@link #purgeableKeys purgeableKeys} LinkedList.  Created mainly for use
	 * in the {@link #resetData() resetData()} function.
	 */
	protected final void resetPurgeableKeys() {
		purgeableKeys = new LinkedList();
	}

	/**
	 * Resets the {@link #data data} to a fresh {@link cdt.wizard.WizardData}.  Only deletes
	 * from the {@link #data data} the keys listed in {@link #purgeableKeys purgeableKeys}
	 * LinkedList.
	 */
	public void resetData() {
		for (int count = 0; count < purgeableKeys.size(); ++count) {
			data.remove((String) purgeableKeys.get(count));
		}
		resetPurgeableKeys();
		//		data = new WizardData();
	}

	/**
	 * Pushs a frame on the stack for later use.
	 * @param frame Frame to push on the stack.
	 */
	public void pushFrame(WizardFrame frame) {
		frames.push(frame);
	}

	/**
	 * Retrieves a frame from the stack.
	 * @return Frame that was on the top of the stack.
	 */
	public WizardFrame popFrame() {
		return (WizardFrame) frames.pop();
	}

	/**
	 * Returns the number of frames currently residing on the stack.
	 *
	 * @return int Number of frames.
	 */
	public int countFrames() {
		return frames.size();
	}

	/**
	 * Removes all frames from the stack.
	 */
	public void purgeFrames() {
		frames = new Stack();
	}

	/**
	 * Enables or disables the cancel button.
	 * @param show <i>true</i> will enable button, <i>false</i> will disable
	 * button.
	 */
	public void showCancelButton(boolean show) {
		cancelButton.setEnabled(show);
	}

	/**
	 * Enables or disables the next button.
	 * @param show <i>true</i> will enable button, <i>false</i> will disable
	 * button.
	 */
	public void showNextButton(boolean show) {
		nextButton.setEnabled(show);
	}

	/**
	 * Enables or disables the previous button.
	 * @param show <i>true</i> will enable button, <i>false</i> will disable
	 * button.
	 */
	public void showPreviousButton(boolean show) {
		previousButton.setEnabled(show);
	}

	/**
	 * Enables or disables the finish button.
	 * @param show <i>true</i> will enable button, <i>false</i> will disable
	 * button.
	 */
	public void showFinishButton(boolean show) {
		finishButton.setEnabled(show);
	}

	/**
	 * Sets the text to be displayed in the top label.
	 * @param newLabel Text to be displayed.
	 */
	public void setFrameLabel1(String newLabel) {
		label1.setText(newLabel);
	}

	/**
	 * Sets the text to be displayed in the bottom label.
	 * @param newLabel Text to be displayed.
	 */
	public void setFrameLabel2(String newLabel) {
		label2.setText(newLabel);
	}

	/**
	 * Handles the push of the cancel button.
	 */
	public void cancelAction() {
		putData("cancelled", "true");
		purgeFrames();
		resetData();
		resetPurgeableKeys(); // This must always be called AFTER resetData()

		setVisible(false);
	}

	/**
	 * Handles the push of the next button.
	 */
	public abstract void nextAction();

	/**
	 *
	 */
	public abstract void customize();

	/**
	 * Handles the push of the previous button.
	 */
	public void previousAction() {
		JDialog previousFrame = popFrame();
		this.setVisible(false); // This setVisible has to come first or it
		// will not disappear right away as expected -B
		previousFrame.setVisible(true);
	}

	/**
	 * Handles the push of the finish button.
	 */
	public void finishAction() {
		putData("canceled", "false");
		purgeFrames();
		resetData();
		resetPurgeableKeys(); // This must always be called AFTER resetData()

		setVisible(false);
	}

	/**
	 * Sends button pushs to correct function.
	 */
	class ButtonHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			if (event.getActionCommand().equals("Cancel")) {
				cancelAction();
			}
			else if (event.getActionCommand().equals("Next >")) {
				nextAction();
			}
			else if (event.getActionCommand().equals("< Previous")) {
				previousAction();
			}
			else if (event.getActionCommand().equals("Finish")) {
				finishAction();
			}
		}
	}
}
